/* A library to test the dynamic linking functionality of guile */

#ifdef NOPRINTS
#define printf(...) /**/
#endif 

static int counter = 0;

/* This function is a "thunk".  It is passed no arguments and returns
   no value. scm_dynamic_call should be able to call this. */

void 
increment_count (void)
{
  counter++;
}

int 
get_count (int argc, char **argv)
{
  return counter;
}

/* These three functions will test the return value functionality of
   scm_dynamic_args_call */

int 
return_zero (int argc, char **argv)
{
  return 0;
}

int 
return_one (int argc, char **argv)
{
  return 1;
}

int 
return_negative_one (int argc, char **argv)
{
  return -1;
}

/* This function will test the passing of information via
   scm_dynamic_args_call.  It returns a 1 on success and a 0 if the
   argc/argv data is inappropriately null or inconsistent. */

int 
count_str_args (int argc, char **argv)
{
  int i;

  printf ("in print_str_array()\n");
  printf ("\t argc = %d\n", argc);
  if (argc > 0) 
    {
      if (argv)
	{
	  for (i = 0; i < argc; i ++)
	    {
	      if (argv[i])
		{
		  printf ("\t %3d: %s\n", i, argv[i]);
		}
	      else
		{
		  printf ("\t argv[%d] shouldn't be NULL\n", i);
		  return -1;
		}
	    }
	}
      else
	{
	  printf ("\t argv shouldn't be NULL\n");
	  return -1;
	}
    }
  else if (argc == 0)
    {
      if (argv)
	{
	  printf ("\t argv is NULL\n");
	}
      else
	{
	  printf  ("\t argv should've been a NULL since argc is 0\n");
	  return -1;
	}
    }

  return argc;
}
	
/* This function will test the passing of information via
   scm_dynamic_args_call.  It expects to receive an string array of
   the form "one" "two" "three".  It returns a 1 on success and a 0 if
   the argc/argv data looks strange */

int 
receive_one_two_three (int argc, char **argv)
{
  int i;
  char test_strings[3][7] = { "one", "two", "three" };

  printf ("in receive_one_two_three()\n");
  printf ("\t argc = %d\n", argc);
  if (argc == 3) 
    {
      if (argv)
	{
	  for (i = 0; i < argc; i ++)
	    {
	      if (argv[i])
		{
		  printf ("\t %3d: %s\n", i, argv[i]);
		}
	      else
		{
		  printf ("\t argv[%d] shouldn't have been NULL\n", i);
		  return 0;
		}
	    }
	  for (i = 0; i < argc; i ++)
	    {
	      if (strcmp (argv[i], test_strings[i]) != 0)
		{
		  printf ("\t argv[%d] doesn't match the test string %s\n", 
			  i, test_strings[i]);
		  return 0;
		}
	    }
	}
      else
	{
	  printf ("\t argv shouldn't have been NULL\n");
	  return 0;
	}
    }
  else
    {
      printf  ("\t argc should've been '3'\n");
      return 0;
    }

  return 1;
}
	
